#include <h5edit.h>

/* Local variables */
static h5edit_debug = 0;	/* debug setting, default no */

void
leave(int ret){
    /* about to leave the tool. Do some necessary cleanup and then quit. */

    /* close all opened files */
    if (commfile_g != NULL)
    	HDfclose(commfile_g);
    /*if (datafile_g >= 0)
	closedatafile(datafile_g);
    */

    /* quit */
    exit(ret);
}


/* Parse command line options.
 * Return if all is okay;
 *     otherwise, print error message and leave(EXIT_FAILURE)
 *     which does not return.
 */
void
parse_option(int ac, const char** av)
{
    int argno;
    const char *s = NULL;

    /* parse in options */
    for (argno = 1; argno < ac && '-' == av[argno][0]; argno++){
	if (streq(av[argno], "--help") || streq(av[argno], "-h")){
	    usage();
	    leave(EXIT_SUCCESS);
	}else if(streq(av[argno], "--dryrun")){
	    dryrun_g = TRUE;
	}else if(streq(av[argno], "--command") || streq(av[argno], "-c")){
	    if (commfilename_g || command_g){	/* --commfile or --commfile option given already */
		fprintf(stderr, "%s: only one of --command or --command-file is allowed\n",
		    progname_g);
		usage();
		leave(EXIT_FAILURE);
	    }
	    if ((argno + 1) >= ac){
		fprintf(stderr, "%s: --command argument missing\n",
		    progname_g);
		usage();
		leave(EXIT_FAILURE);
	    }
	    command_g = av[++argno];
	}else if(streq(av[argno], "--command-file")){
	    if (commfilename_g || command_g){	/* --commfile or --commfile option given already */
		fprintf(stderr, "%s: only one of --command or --command-file is allowed\n",
		    progname_g);
		usage();
		leave(EXIT_FAILURE);
	    }
	    if ((argno + 1) >= ac){
		fprintf(stderr, "%s: --command-file argument missing\n",
		    progname_g);
		usage();
		leave(EXIT_FAILURE);
	    }
	    commfilename_g = av[++argno];
	    if ((commfile_g=opencommfile(commfilename_g))==NULL){
		perror(progname_g);
		fprintf(stderr, "%s: could not open command file (%s)\n",
		    progname_g, commfilename_g);
		usage();
		leave(EXIT_FAILURE);
	    }
	}else if(streq(av[argno], "--version")){
	    print_version();
	    leave(EXIT_SUCCESS);
	} else {
	    fprintf(stderr, "%s: Unknown option (%s)\n",
		progname_g, av[argno]);
	    usage();
	    leave(EXIT_FAILURE);
	}
    } /* end for */

    /* check if command or command-file is provided. */
    if (command_g==NULL && commfilename_g==NULL){
	fprintf(stderr, "%s: missing command argument\n", progname_g);
	usage();
	leave(EXIT_FAILURE);
    }

    /* look for datafile argument */
    if (argno >= ac){
	fprintf(stderr, "%s: missing datafile argument\n", progname_g);
	usage();
	leave(EXIT_FAILURE);
    }
    datafilename_g = av[argno++];
    
    if ((datafile_g=opendatafile(datafilename_g)) < 0){
	perror(progname_g);
	fprintf(stderr, "%s: could not open data file (%s)\n",
	    progname_g, datafilename_g);
	usage();
	leave(EXIT_FAILURE);
    }

    /* Turn off HDF5's automatic error printing. */
    /* Should have an option to allow automatic error printing for debugging. */
    if (H5Eset_auto2(H5E_DEFAULT, NULL, NULL) < 0){
	fprintf(stderr, "%s: failed to turn off automatic error printing\n", progname_g);
	leave(EXIT_FAILURE);
    }

    return;
}

void
usage(void)
{
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "%s [-h | --help] h5file\n", progname_g);
    fprintf(stderr, "%s [OPTIONS] h5file\n", progname_g);
    fprintf(stderr, 
	"OPTIONS\n" \
	"   -h, --help\n"
	"       Print a usage message and exit\n"
	"   -c command, --command command:\n"
	"       Specify an H5edit command to apply to the file h5file.\n"
	"   --command-file commfile\n"
	"       Specify a command file, commfile, that contains H5edit commands written\n"
	"       in the H5edit Command Language, to apply to the file h5file.\n"
	"   --dryrun\n"
	"       (To be supported in future implementation)\n"
	"       Just check the syntax of the H5edit commands against the HDF5 file\n"
	"       without making the actual changes to the HDF5 file.\n"
	"   --version\n"
	"       Print the version information of h5edit.\n"
	);
    fprintf(stderr, "\n");

} /* usage() */


/* In the future, data file is open with atomicity in mind. For now, just
 * H5Fopen it.
 */
hid_t opendatafile(const char *name){
    hid_t ret_code;

    if ((ret_code=H5Fopen(name, H5F_ACC_RDWR, H5P_DEFAULT))<0){
	fprintf(stderr, "%s: could not open data file (%s)\n",
		progname_g, name);
	leave(EXIT_FAILURE);
    };
    return(ret_code);
}

/* Open the command file */
FILE *opencommfile(const char *name){
    FILE *ret_code;

    if ((ret_code=HDfopen(name, "r"))==NULL){
	fprintf(stderr, "%s: could not open command file (%s)\n",
		progname_g, name);
	leave(EXIT_FAILURE);
    };
    return(ret_code);
}

/*-------------------------------------------------------------------------
 * Execute h5edit commands.
 * For now, no return value--always succeed.
 * If error is encountered, it would have gone to exit function leave()
 * and exit from there.
 # This sets up the input stream, first from command-file if given.  If not,
 * setup from command string.
 */
void execute_command(void)
{
    /* setup command file input if given */
    if (commfile_g != NULL){
	h5edit_commfile = commfile_g;
	h5edit_command = NULL;
	h5edit_input_len = -1;
    } else {
	h5edit_commfile = NULL;
	h5edit_command = command_g;
	h5edit_input_len = strlen(h5edit_command);
    }

    if(H5EDITyyparse() != 0)
        goto quit;

    /* Normal return. */
    /* Keeping this in case we want to distinguish it from quit */
    h5edit_commfile = NULL;
    h5edit_command = NULL;
    h5edit_input_len = -1;
    return;

quit:
    h5edit_commfile = NULL;
    h5edit_command = NULL;
    h5edit_input_len = -1;
    return;
}


/* Set debug varilable if environment variable $H5EDIT_DEBUG is set */
void
set_debug(void)
{
    if (HDgetenv("H5EDIT_DEBUG") != NULL){
	h5edit_debug = 1;
	printf("debug set to 1\n");
    }
}

/* return if debug is on.
 * TRUE (not 0) if set;
 * FALSE (0) otherwise.
 */
int
get_debug(void)
{
    return(h5edit_debug);
}

    
/* Print the version information of h5edit */
void
print_version(void)
{
    fprintf(stdout, H5EDIT_VERSION_FORMAT_STRING "\n",
	H5EDIT_VER_MAJOR, H5EDIT_VER_MINOR, H5EDIT_VER_RELEASE,
	H5EDIT_RELEASE_DATE);
}

